/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.mojo;

import java.io.File;
import java.io.IOException;
import java.util.List;

import jp.objectfanatics.assertion.constraints.NotNull;
import jp.objectfanatics.assertion.weaver.api.exception.IllegalUseOfConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.exception.NoWeavingNeededException;
import jp.objectfanatics.assertion.weaver.api.exception.NotClassFileException;
import jp.objectfanatics.assertion.weaver.api.exception.info.BehaviorInfo;
import jp.objectfanatics.assertion.weaver.api.exception.info.TypeInfo.PackageNotFoundException;
import jp.objectfanatics.assertion.weaver.impl.core.ClassFolderWeaver;
import jp.objectfanatics.assertion.weaver.impl.core.ClassFolderWeaver.WeavingProgressListener;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * reference
 * <ul>
 *   <li>http://maven.apache.org/plugin-developers/index.html
 *   <li>http://maven.apache.org/guides/introduction/introduction-to-the-lifecycle.html
 * </ul>
 * 
 * @author Makoto Sato
 */
abstract public class AbstractWeavingMojo extends AbstractMojo {
	
	/**
	 * Directory containing the build files.
	 * 
	 * @parameter expression="${project.build.directory}"
	 */
	protected File buildDirectory;
	
	/**
	 * Root path to be woven.
	 */
	protected String path;
	
	/**
	 * Source root path. It may be null.
	 */
	protected String sourceRoot;
	
	@Override
	public void execute() throws MojoExecutionException {
		try {
			String[] classpaths = toArray(getClasspaths());
			getLog().info("start weaving " + path);
			new ClassFolderWeaver(classpaths).weave(path, createClassFolderWeaverListener());
			getLog().info("end   weaving " + path);
		}
		catch (IOException e) {
			throw new MojoExecutionException(e.getMessage(), e);
		}
		catch (IllegalUseOfConstraintAnnotationException e) {
			throw new MojoExecutionException(e.getMessage(), e);
		}
	}
	
	private WeavingProgressListener createClassFolderWeaverListener() {
		return new WeavingProgressListener() {

			@Override
			public void notified(File classFile, NotClassFileException e) {
				getLog().debug(" - skip : " + classFile.getAbsolutePath());
			}
			
			@Override
			public void notified(File classFile, NoWeavingNeededException e) {
				getLog().debug(" - no weaving needed : " + classFile.getAbsolutePath());
			}
			@Override
			public void notified(File classFile, IllegalUseOfConstraintAnnotationException e) {
				String[] lines = e.getDetailedDescription().split("\\n");
				for (int i=0; i<lines.length; i++) {
					if (i == 0) {
						getLog().info(" - " + lines[i]);
						getLog().info("   source file -> " + createSourceFileName(e.getTargetBehavior()));
						getLog().info("   class  file -> " + classFile.getAbsolutePath());
					}
					else {
						getLog().info("   " + lines[i]);
					}
				}
			}
		};
	}
	
	/**
	 * Returns classpaths
	 * 
	 * @return classpaths
	 */
	abstract protected List<String> getClasspaths();
	
	/**
	 * Converts List<String> to String[].
	 * 
	 * @param list
	 *            list
	 * @return string array
	 */
	@NotNull
	private String[] toArray(@NotNull List<String> list) {
		String[] classpaths = new String[list.size()];
		for (int i = 0; i < list.size(); i++) {
			classpaths[i] = list.get(i);
		}
		return classpaths;
	}
	
	private String createSourceFileName(BehaviorInfo behaviorInfo) {
		if (sourceRoot == null) {
			return behaviorInfo.getSourceFileName();
		}
		else {
			return sourceRoot + File.separator + createPackagePath(behaviorInfo) + behaviorInfo.getSourceFileName();
		}
	}

	private String createPackagePath(BehaviorInfo behaviorInfo) {
		String packagePath;
		try {
			packagePath = behaviorInfo.getPackageName().replace(".", File.separator) + File.separator;
		}
		catch (PackageNotFoundException e) {
			packagePath = "";
		}
		return packagePath;
	}
}
